/**
 * KeyMap.java
 * 
 * Maps the user's keys to input codes and tracks which keys are being pressed
 * 
 * @author mmasukaw
 */

package inf119;

import java.awt.event.KeyEvent;
import inf119.MovementType;

public class KeyMap {
	private int[] keyMaps;
	private int[] menuKeys;
	private int[] menuMaps;
	private int[] cameraMaps;
	
	public static final int FIRST_PERSON_CAM = 0, THIRD_PERSON_CAM = 1, FOLLOW_CAM = 2;
	private int camMode = THIRD_PERSON_CAM;
	private boolean followCam = false;
	
	private boolean[] movements;
	private final int NUM_MENU_KEYS = 1;
	
	/* 
	 * If movements[i] is true, then the user is holding down control key i
	 * However: if the user releases that key in such a way that the KeyEvent is 
	 * not reported to our listener then our code cannot know about it. 
	 */
	
	public KeyMap()
	{
		 movements = new boolean[MovementType.numValues()];
		 keyMaps = new int[MovementType.numValues() + 1];

		 /* This is where movement control keys are defined */
		 mapKey(MovementType.YAW_LEFT, KeyEvent.VK_Q);
		 mapKey(MovementType.YAW_RIGHT, KeyEvent.VK_E);
		 mapKey(MovementType.PITCH_LEFT, KeyEvent.VK_S);
		 mapKey(MovementType.PITCH_RIGHT, KeyEvent.VK_W);
		 mapKey(MovementType.ROLL_LEFT, KeyEvent.VK_A);
		 mapKey(MovementType.ROLL_RIGHT, KeyEvent.VK_D);
		 mapKey(MovementType.THRUST, KeyEvent.VK_SPACE);
	 
		 
		 //TODO: do this with a more elegant data structure, perhaps a hash table
		 /* Map the various menus to their corresponding keys */
		 menuKeys = new int[NUM_MENU_KEYS];
		 menuMaps = new int[NUM_MENU_KEYS];
		 
		 /* Map the menu to it's corresponding key */
		 menuKeys[0] = KeyEvent.VK_P;
		 menuMaps[0] = 1; //the value of MyxMenuController.PAUSE_MENU
		 
		 /* Map miscellaneous system keys */
		 cameraMaps = new int[FOLLOW_CAM + 1]; //Make sure this gets updated if there are more modes
		 cameraMaps[FIRST_PERSON_CAM] = KeyEvent.VK_F1;
		 cameraMaps[THIRD_PERSON_CAM] = KeyEvent.VK_F3;
		 cameraMaps[FOLLOW_CAM] = KeyEvent.VK_F4;
	}
	
	public int getMenuCode(int code)
	{
		for (int i = 0; i < menuKeys.length; i++){
			if (menuKeys[i] == code)
				return menuMaps[i];
		}
		return 2; //the value of MyxMenuController.NO_CHANGE
	}
	
	/* This function lets you remap movement control keys */
	public void mapKey(MovementType mt, int code)
	{
		keyMaps[mt.toInt()] = code;
	}

	public void keyPressed(int code)
	{
		for (int i = 0; i < keyMaps.length; i++){
			if (keyMaps[i] == code){
				movements[i] = true;
			}
		}
		if (code == cameraMaps[FIRST_PERSON_CAM]){
			//Switch to first person camera mode
			camMode = FIRST_PERSON_CAM;
		}
		else if (code == cameraMaps[THIRD_PERSON_CAM]){
			//Switch to third person camera mode
			camMode = THIRD_PERSON_CAM;
		}
		else if (code == cameraMaps[FOLLOW_CAM]){
			//Toggle Follow-Cam
			followCam = !followCam;
		}	
	}
	
	public void keyReleased(int code)
	{
		for (int i = 0; i < keyMaps.length; i++){
			if (keyMaps[i] == code)
				movements[i] = false;
		}		
	}
	
	public boolean[] getPressedKeys()
	{
		//NOTE: this is fast but not necessarily safe, maybe we should make a copy?
		return movements;
	}
	public int getCameraMode()
	{
		return camMode;
	}
	
	public boolean isFollow()
	{
		return followCam;
	}
}
