package inf119;

public class LanderModel {

	public double resolution;	//distance in meters between vertices
	public int[] textureIDs;	//textures for lander model
	public final static int  MAX_TEX = 2; //cap number of textures for model until we change the way we deal with textures
		
	private double[][][] vertices; //model vertices specified in polygon groups
	// NOTE:  faces with first specified texture should be specified first in vertices[][][]
	public int texFaces; // indicates how many faces use the first texture
	
	public String name;    //Lander model name, e.g. "Mars Polar Lander"
	public String description;   //brief description of the model type and features
	public double footHeight;	//distance from foot base of model object
	public double thrustLoc[];	//x, y, z coords for thruster center, offset from centMass
	public double thrustRad;	//radius of thruster
	public double centMass[];  //center of mass of the lander
	
	private double width, height; //dimensions of the lander
	
	/*
	 * Constructor
	 */
	public LanderModel(double res, int[] texID, double[][][] verts, int tex1, double[] thrustSpot, double thrustRadius, double footDist, double[] mass )
	//sets the data for drawing a lander model (with the specified textures)
	{
		resolution = res;
		vertices = verts;
		texFaces = tex1;
		thrustLoc = thrustSpot;
		thrustRad = thrustRadius;
		footHeight = footDist;
		centMass = mass;
		
		if( texID.length < MAX_TEX )
			textureIDs = texID;
		else {
			//hardcoded by default
			textureIDs = new int[2];
			textureIDs[0] = 2;  //this texture applied to faces specified in texFaces
			textureIDs[1] = 3;  //this texture will be applied to all other faces 
		}
		
		calcHeight();
		calcWidth();	
		
	}
	
	public void setVertices(double[][][] verts)
	{
		vertices = verts;
	}
	
	public double[][][] getVertices()
	{
		return vertices;
	}
	
	private void calcWidth()
	{
		//assumes uniform width, i.e., width in x direction equals width in y direction
		
		double big = Math.abs(vertices[0][0][2]);
		for( int i = 0; i < vertices.length; i++ )
		{
			for( int j = 0; j < 3; j++ )
			{
				double temp = Math.abs(vertices[i][j][2]);
			
				if( big < temp )
					big = temp;				
			}
		}
		
		width = big*2;
	}
	
	private void calcHeight()
	{
		double highest = Math.abs(vertices[0][0][1]);
		for( int i = 0; i < vertices.length; i++ )
		{
			for( int j = 0; j < 3; j++ )
			{
				double temp = Math.abs(vertices[i][j][1]);
				if(highest < temp)
					highest = temp;
			}
		}
			
		height = highest*2;
	}
	
	public double getWidth()
	{
		return resolution*width;
	}
	
	public double getHeight()
	{
		return resolution*height;
	}
	
	
}
