package inf119.gamelogic;

import java.awt.Font;

import edu.uci.isr.myx.fw.AbstractMyxSimpleBrick;
import edu.uci.isr.myx.fw.IMyxName;
import edu.uci.isr.myx.fw.MyxUtils;

import inf119.Constants;
import inf119.interfaces.MenuControllerIntf;
import inf119.interfaces.StateIntf;

public class MyxMenuController extends AbstractMyxSimpleBrick implements MenuControllerIntf {

	public static final IMyxName msg = MyxUtils.createName("MenuController");
	public static final IMyxName statemsg = MyxUtils.createName("state");
	
	//Codes for menu control.  Note: NO_CHANGE is used in getNumMenus()
	public static final int CLEAR_MENU = 0, PAUSE_MENU = 1, NO_CHANGE = 2;
	
	StateIntf GSC;
	Menu menu;
	Font font;
	
	public static int getNumMenus()
	{
		return NO_CHANGE;
	}
	
	public MyxMenuController ()
	{
		font = new Font("SansSerif", Font.PLAIN, 14);				
	}
	
	public void begin()
	{
		GSC = (StateIntf)MyxUtils.getFirstRequiredServiceObject(this, statemsg);		
	}
	
	public void setMenu(int menuType)
	{
		switch (menuType){
		case (CLEAR_MENU): clearMenu(); break;
		case (PAUSE_MENU): menu = getPauseMenu();
		}
		GSC.setMenu(menu);
	}
	
	public boolean checkMenuOption(int code)
	//Returns true if there is an active menu
	//Returns false otherwise
	{
		menu = GSC.getMenu();
		
		if (menu == null){
			return false;
		}
		MenuItem[] items = menu.getMenuItems();
		for (int i = 0; i < items.length; i++){
			if (items[i].getCode() == code){
				GSC.updateGameState(items[i].getNextState());
				clearMenu();
			}
		}
		return true;
	}
	
	public void setFont(Font f)
	{
		font = f;
	}
	
	private Menu getPauseMenu()
	{
		MenuItem[] items = new MenuItem[2];
		items[0] = new MenuItem('R', "Resume Game", Menu.LEFT_ALIGN, Constants.LIVEGAMESTATE);
		items[1] = new MenuItem('Q', "Quit Game", Menu.LEFT_ALIGN, Constants.QUITGAMESTATE);
		
		return new Menu("Pause", Menu.CENTER_ALIGN, items, font);
	}
	
	public void clearMenu()
	{
		menu = null;
		GSC.setMenu(menu);
	}

	public Object getServiceObject(IMyxName arg0) {
		// TODO Auto-generated method stub
		if (arg0.equals(msg)){
			return this;
		}
		return null;
	}
}
