package inf119.gamelogic;

import edu.uci.isr.myx.fw.AbstractMyxSimpleBrick;
import edu.uci.isr.myx.fw.IMyxName;
import edu.uci.isr.myx.fw.MyxUtils;
import inf119.MovementType;
import inf119.interfaces.PhysicsModelIntf;
import inf119.interfaces.StateIntf;

public class MyxPhysicsModel extends AbstractMyxSimpleBrick implements PhysicsModelIntf {

	public static final IMyxName msg = MyxUtils.createName("PhysicsModel");
	public static final IMyxName statemsg = MyxUtils.createName("state");
	
	//These Constants map indicies to the Arrays given to us via GameStateConnector
	//Ship Indicies
	private  final int fuel_amount 	= 0;
	private  final int x_speed 		= 1;
	private  final int y_speed 		= 2;
	private  final int z_speed 		= 3;
	private  final int x_position	= 4;
	private  final int y_position	= 5;
	private  final int z_position	= 6;
	private  final int Pitch_Angle	= 7;
	private  final int Roll_Angle 	= 8;
	private  final int Yaw_Angle	= 9;
	private  final int Pitch_Omega 	=10;
	private  final int Roll_Omega 	=11;
	private  final int Yaw_Omega 	=12;

	//Environment Indicies
	private  final int planet_gravity =0;
	private  final int thruster_force=1;
	private  final int main_thruster=2;
	private  final int ship_mass=3;
	private  final int ship_radius=4;
	private  final int planet_radius=5;
	private  final int ship_height=6;
	private  final int burn_rate=7;

	//These are the Variables that get modified and used throughout the program
	//private  double[] Ship_Variables_Array = {1000,0.0,0.0,0.0,0.0,1752000.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0};
	//private  double[] Environment_Constants_Array = {1.6, 75000, 1400500,300000, 2.1, 1738000.0,8.0,5.0};
	private  double[] Ship_Variables_Array;
	private  double[] Environment_Constants_Array;
	private  StateIntf GameState;
	private  final double x_position_head = 0.0000001;
	private  final double y_position_head = 4.0000001;
	private  final double z_position_head = 0.0000001;
	private  double x_position_head_new = 0.0;
	private  double y_position_head_new = 0.0;
	private  double z_position_head_new = 0.0;
	private  double Yaw_Accel = 0.0;
	private  double Pitch_Accel = 0.0;
	private  double Roll_Accel = 0.0;
	private  double x_Force = 0.0;
	private  double y_Force = 0.0;
	private  double z_Force = 0.0;
	private  double x_accel = 0.0;
	private  double y_accel = 0.0;
	private  double z_accel = 0.0;

	public void begin(){
		GameState = (StateIntf)MyxUtils.getFirstRequiredServiceObject(this, statemsg);
		GameState.begin();
		Environment_Constants_Array = GameState.getEnvironmentConstants();
		Ship_Variables_Array = GameState.getShipVariables();
	}

	private void UpdateData()
	{
		Ship_Variables_Array = GameState.getShipVariables();
		Environment_Constants_Array = GameState.getEnvironmentConstants();
	}


	//Function used to round the double value to a certain precision
	private  double Round(double input, int precision)
	{
		double temp = input * Math.pow(10.0, precision);
		temp = Math.round(temp);
		return (temp / Math.pow(10.0, precision));
	}

	//Function is in Call to Physics
	//Given a thruster, it will calculate the new orientation of the ship
	public  void Thrust(MovementType movement,double miliseconds)
	{
		UpdateData();
		double seconds = miliseconds/1000;
		RotationalAcceleration(movement);
		//RotationalDisplacement(movement,seconds);
		RotationalVelocityChange(movement,seconds);
		ShipOrientation(movement);
		SetGameStateVariables();
		//PrintOutChange(movement.toString());
		//PrintVariables();
		SetLocalVariables();
		Ship_Variables_Array[fuel_amount] -= Environment_Constants_Array[burn_rate];
	}

	//Function is in Call to Physics
	//When Main Thruster is pressed, this will put the ship in it's new position
	public  void MaxThrust(double miliseconds)
	{
		UpdateData();
		double seconds = miliseconds/1000;
		ContinueRotation(seconds);
//		System.out.println("Yaw Angle "+Ship_Variables_Array[Yaw_Angle]);
//		System.out.println("Pitch Angle "+Ship_Variables_Array[Pitch_Angle]);
//		System.out.println("Roll Angle "+Ship_Variables_Array[Roll_Angle]);
		ShipOrientation(MovementType.YAW_LEFT);
		ShipOrientation(MovementType.PITCH_LEFT);
		ShipOrientation(MovementType.ROLL_LEFT);
		DirectionalForce(Environment_Constants_Array[main_thruster]);
		DirectionalAcceleration();
		DirectionalSpeed(seconds);
		NewPosition(seconds);
		Ship_Variables_Array[fuel_amount] -= 2*Environment_Constants_Array[burn_rate];
		SetGameStateVariables();
		//PrintOutChange("Thrust");
		//PrintVariables();
		SetLocalVariables();
		
	}
	//Function is in Call to Physics
	//Game Logic calls this in order to tell physics that time has passed and gravity needs to come into play
	public  void TimePass(double miliseconds)
	{
		UpdateData();
		double seconds = miliseconds/1000;
		ContinueRotation(seconds);
		ShipOrientation(MovementType.YAW_LEFT);
		ShipOrientation(MovementType.PITCH_LEFT);
		ShipOrientation(MovementType.ROLL_LEFT);
		DirectionalForce(0.0);
		DirectionalAcceleration();
		DirectionalSpeed(seconds);
		NewPosition(seconds);
		SetGameStateVariables();
		//PrintOutChange("Time Pass");
		//PrintVariables();
		SetLocalVariables();
	}

	//Calculates the angular acceleration based on which thruster is turned on
	//uses Force and Radius to get Torque, then uses torque and Inertia to determine acceleration
	//T = F*r
	//I = (1/12)*m(3r^2+l^2)     or I=.5mr^2
	//T = I*a
	private  void RotationalAcceleration(MovementType movement)
	{
		double radius = Environment_Constants_Array[ship_radius];
		double mass = Environment_Constants_Array[ship_mass]+Ship_Variables_Array[fuel_amount];
		double Torque = 0.0;
		double Inertia = 0.0;

		if (movement == MovementType.YAW_LEFT || movement == MovementType.PITCH_LEFT || movement == MovementType.YAW_RIGHT || movement == MovementType.PITCH_RIGHT)
		{
			double length = Environment_Constants_Array[ship_height]/2;
			Torque = Environment_Constants_Array[thruster_force]*radius;
			Inertia = (mass*(3*radius*radius+length*length))/12;


			if(movement == MovementType.YAW_LEFT)
				Yaw_Accel = (-1)*Torque/Inertia;
			else if(movement == MovementType.YAW_RIGHT)
				Yaw_Accel = Torque/Inertia;
			else if(movement == MovementType.PITCH_LEFT)
				Pitch_Accel = (-1)*Torque/Inertia;
			else if(movement == MovementType.PITCH_RIGHT)
				Pitch_Accel = Torque/Inertia;
		}
		else if(movement == MovementType.ROLL_LEFT || movement == MovementType.ROLL_RIGHT)
		{
			Torque = Environment_Constants_Array[thruster_force]*radius;
			Inertia = .5*mass*radius*radius;

			if(movement == MovementType.ROLL_LEFT)
				Roll_Accel = (-1)*Torque/Inertia;
			else if(movement == MovementType.ROLL_RIGHT)
				Roll_Accel = Torque/Inertia;
		}
	}


	//Calculates the Displacement in radians given the current orientation and new thruster
	//xf = xi + vt + .5at^2
	private  void RotationalDisplacement(MovementType movement, double seconds)
	{
		if(movement == MovementType.YAW_LEFT || movement == MovementType.YAW_RIGHT)
		{
			Ship_Variables_Array[Yaw_Angle] = Round(Ship_Variables_Array[Yaw_Angle] + Ship_Variables_Array[Yaw_Omega]*seconds + .5*Yaw_Accel*seconds*seconds,4);;
		}
		else if(movement == MovementType.PITCH_RIGHT || movement == MovementType.PITCH_LEFT)
		{
			Ship_Variables_Array[Pitch_Angle] = Round(Ship_Variables_Array[Pitch_Angle] + Ship_Variables_Array[Pitch_Omega]*seconds + .5*Pitch_Accel*seconds*seconds,4);;
		}
		else if(movement == MovementType.ROLL_LEFT || movement == MovementType.ROLL_RIGHT)
		{
			Ship_Variables_Array[Roll_Angle] = Round(Ship_Variables_Array[Roll_Angle]+Ship_Variables_Array[Roll_Omega]*seconds + .5*Roll_Accel*seconds*seconds,4);;
		}


	}

	//Calculates the Angular Velocity and saves it to the variable array
	//vf = vi + at
	private  void RotationalVelocityChange(MovementType movement,double seconds)
	{
		if(movement == MovementType.YAW_LEFT || movement == MovementType.YAW_RIGHT)
		{
			Ship_Variables_Array[Yaw_Omega]=Round(Ship_Variables_Array[Yaw_Omega]+Yaw_Accel*seconds,4);
		}
		else if(movement == MovementType.PITCH_RIGHT || movement == MovementType.PITCH_LEFT)
		{
			Ship_Variables_Array[Pitch_Omega]=Round(Ship_Variables_Array[Pitch_Omega]+ Pitch_Accel*seconds,4);
		}
		else if(movement == MovementType.ROLL_LEFT || movement == MovementType.ROLL_RIGHT)
		{
			Ship_Variables_Array[Roll_Omega]=Round(Ship_Variables_Array[Roll_Omega]+Roll_Accel*seconds,4);
		}


	}

	//Uses Rotational Matrix to determine the orientation of the ship and sets the location where the head of the ship is pointing
	private  void ShipOrientation(MovementType movement)
	{
		if((movement == MovementType.ROLL_LEFT || movement == MovementType.ROLL_RIGHT)&&Ship_Variables_Array[Roll_Angle]!=0.0)
		{
			//y_position_head_new = 0.0;
			x_position_head_new = Round(x_position_head*Math.cos(Ship_Variables_Array[Roll_Angle]) - z_position_head*Math.sin(Ship_Variables_Array[Roll_Angle]),4);
			z_position_head_new = Round(z_position_head*Math.cos(Ship_Variables_Array[Roll_Angle]) + x_position_head*Math.sin(Ship_Variables_Array[Roll_Angle]),4);
		}
		else if((movement == MovementType.PITCH_RIGHT || movement == MovementType.PITCH_LEFT)&&Ship_Variables_Array[Pitch_Angle]!=0.0)
		{
			y_position_head_new = Round(y_position_head*Math.cos(Ship_Variables_Array[Pitch_Angle]) + z_position_head*Math.sin(Ship_Variables_Array[Pitch_Angle]),4);
			z_position_head_new = Round(z_position_head*Math.cos(Ship_Variables_Array[Pitch_Angle]) - y_position_head*Math.sin(Ship_Variables_Array[Pitch_Angle]),4);
		}
		else if((movement == MovementType.YAW_LEFT || movement == MovementType.YAW_RIGHT)&&Ship_Variables_Array[Yaw_Angle]!=0.0)
		{
			x_position_head_new = Round(x_position_head*Math.cos(Ship_Variables_Array[Yaw_Angle]) + y_position_head*Math.sin(Ship_Variables_Array[Yaw_Angle]),4);
			y_position_head_new = Round(y_position_head*Math.cos(Ship_Variables_Array[Yaw_Angle]) - x_position_head*Math.sin(Ship_Variables_Array[Yaw_Angle]),4);
		}
	}

	//Used in Time Pass, will determine the angle at which the ship has rotated given the time that has passed
	//xf = xi + vt      same as xf = xi + vt + .5at^2 but acceleration is equal to zero
	private  void ContinueRotation(double time)
	{
		if(Ship_Variables_Array[Yaw_Omega]!=0.0)
		{
			Ship_Variables_Array[Yaw_Angle] = Ship_Variables_Array[Yaw_Angle] + Ship_Variables_Array[Yaw_Omega]*time;
		}
		if(Ship_Variables_Array[Pitch_Omega]!=0.0)
		{
			Ship_Variables_Array[Pitch_Angle] = Ship_Variables_Array[Pitch_Angle] + Ship_Variables_Array[Pitch_Omega]*time;
		}
		if(Ship_Variables_Array[Roll_Omega]!=0.0)
		{
			Ship_Variables_Array[Roll_Angle] = Ship_Variables_Array[Roll_Angle] + Ship_Variables_Array[Roll_Omega]*time;
		}
	}

	//Finds the force in each of the 3 dimensions that will be applied when the main thruster is hit depending on what angles the ship is in
	private  void	DirectionalForce(double Force)
	{
		double length = Environment_Constants_Array[ship_height]/2;
		x_Force = Force*x_position_head_new/length;
		y_Force = Force*y_position_head_new/length - Environment_Constants_Array[planet_gravity]*(Environment_Constants_Array[ship_mass]+Ship_Variables_Array[fuel_amount]);
		z_Force = Force*z_position_head_new/length;
	}

	//Since there will be a force in each of the three dimensions there will be acceleration as well
	//F=ma
	private  void DirectionalAcceleration()
	{
		double mass = Environment_Constants_Array[ship_mass]+Ship_Variables_Array[fuel_amount];
		x_accel = x_Force / (mass);
		y_accel = y_Force / (mass);
		z_accel = z_Force / (mass);
	}
	//This will determine the spead of the ship in the 3 dimensions
	//vf = vi + at
	private  void DirectionalSpeed(double time)
	{
		Ship_Variables_Array[x_speed] = Ship_Variables_Array[x_speed] + x_accel*time;
		Ship_Variables_Array[y_speed] = Ship_Variables_Array[y_speed] + y_accel*time;
		Ship_Variables_Array[z_speed] = Ship_Variables_Array[z_speed] + z_accel*time;
	}
	//This will determine the position of the ship relative to the planet after the main thruster has been applied
	//xf = xi + vt + .5at^2
	private  void NewPosition(double time)
	{
		Ship_Variables_Array[x_position] = Round(Ship_Variables_Array[x_position] + Ship_Variables_Array[x_speed]*time + .5*x_accel*time*time,4);
		Ship_Variables_Array[y_position] = Round(Ship_Variables_Array[y_position] + Ship_Variables_Array[y_speed]*time + .5*x_accel*time*time,4);
		Ship_Variables_Array[z_position] = Round(Ship_Variables_Array[z_position] + Ship_Variables_Array[z_speed]*time + .5*x_accel*time*time,4);
	}

	//Call to GameStateConnector so others can use the updated variables
	private  void SetGameStateVariables()
	{
		GameState.setShipVariables(Ship_Variables_Array);
	}

	//sets local variables back to zero
	private  void SetLocalVariables()
	{
		x_position_head_new = 0.0;
		y_position_head_new = 4.0;
		z_position_head_new = 0.0;
		Yaw_Accel = 0.0;
		Pitch_Accel = 0.0;
		Roll_Accel = 0.0;
		x_Force = 0.0;
		y_Force = 0.0;
		z_Force = 0.0;
		x_accel = 0.0;
		y_accel = 0.0;
		z_accel = 0.0;
	}

	public Object getServiceObject(IMyxName arg0) {
		// TODO Auto-generated method stub
		if (arg0.equals(msg)){
			return this;
		}
		return null;
	}
}
