package inf119.gamestate;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.StringTokenizer;

import edu.uci.isr.myx.fw.AbstractMyxSimpleBrick;
import edu.uci.isr.myx.fw.IMyxName;
import edu.uci.isr.myx.fw.MyxUtils;

import inf119.TerrainMap;
import inf119.interfaces.EnvironmentDataIntf;
import inf119.gamelogic.Menu;

public class MyxEnvironmentData extends AbstractMyxSimpleBrick implements EnvironmentDataIntf {
	
	public static final IMyxName msg = MyxUtils.createName("EnvironmentData");
	
	private String planet_name; //Planet's name
	private String planetPictureFileName; //Picture file name that corresponds to planet
	private double gravity; //Planet's gravity
	private double radius; //Planet's radius

	private int number_of_environ_vars; //Number of lines in planet file
	private int numberOfNonTokenizedLines; //Number of non tokenized lines in planet file
	private String[] environ; //String array of contents of planet file
	private String defaultplanet; //Name of default planet that is loaded on creation of class
	
	//Variables used to create the Terrain Map
	private TerrainMap tMap; //TerrainMap object
	private int mapWidth; //Map width
	private int mapLength; //Map height
	private int mapResolution;// map resolution
	private double[][] mapHeights; //2D of tokenized heights
	
	private int numberOfLandingSites; //Number of landing sites
	private int numberOfObjects; //Number of objects on surface of planet
	private int[][] landingSites; //2D array of landing sites and their respective values
	private int[][] objects; //2D array of objects and their respective values
	
	private int textureID; //Texture of the surface of the planet
	private int sizeOfLandingSitesArray; //Number of variables each landing site has
	private int sizeOfObjectsArray; //Number of variables each object on the planet has
	
	
	private Menu inGameMenu;
	//Sets variables and loads default planet
	public MyxEnvironmentData() throws FileNotFoundException, IOException
	{
		
		//Reads in planet file		
		FileRead environ_file = new FileRead();
		defaultplanet = "defaultplanet.planet";
		number_of_environ_vars = environ_file.numberOfLinesInFile(defaultplanet);
		
		sizeOfLandingSitesArray =4;
		sizeOfObjectsArray = 3;
		numberOfNonTokenizedLines = 7;
		
		environ = new String[number_of_environ_vars];		
		environ = environ_file.readFile(defaultplanet, "planet", number_of_environ_vars);
				
		convertArrayIntoVars(); //Coverts string array into usable vars

	}
	
	
	//Loads a new planet
	public void changePlanet(String fileName) throws FileNotFoundException, IOException
	{
		FileRead environ_file = new FileRead();
		
		number_of_environ_vars = environ_file.numberOfLinesInFile(fileName);
		
		environ = new String[number_of_environ_vars];
		environ = environ_file.readFile(fileName, "planet", number_of_environ_vars);
		convertArrayIntoVars();
		
	}
	
	//Converts string array into proper values
	private void convertArrayIntoVars()
	{
		//Sets non tokenized variables
		planet_name = environ[0];
		planetPictureFileName = environ[1];
		gravity = Double.parseDouble(environ[2]);
		radius = Double.parseDouble(environ[3]);	
		textureID = Integer.parseInt(environ[4]);
		numberOfLandingSites = Integer.parseInt(environ[5]);
		numberOfObjects = Integer.parseInt(environ[6]);
		
		//Sets size of landing sites and planet objects
		landingSites = new int[numberOfLandingSites][sizeOfLandingSitesArray];
		objects = new int[numberOfObjects][sizeOfObjectsArray];	
		
		//Sets variables to the end of landing sites, objects and map height header
		//Done so that each for loop reads only appropriate info
		int endOfLZs = numberOfNonTokenizedLines + numberOfLandingSites; //Gives the array cell number right after the end of the landing sites 
		int endOfObjs = endOfLZs + numberOfObjects; //Gives the array cell number right after the end of the objects 
		int endHeightHeader = endOfObjs +1; //Gives the array cell number right after the end of the map height header
				
		for(int i=numberOfNonTokenizedLines; i<endOfLZs ;i++) //Converts landing sites
		{
			String tempString = environ[i];			
			StringTokenizer st = new StringTokenizer(tempString);
			
			for(int j=0; j<sizeOfLandingSitesArray;j++)
			{	
				landingSites[i-numberOfNonTokenizedLines][j] = Integer.parseInt(st.nextToken());
			}

		}
		
		for(int i=endOfLZs; i<endOfObjs ;i++) //Converts objects
		{
			String tempString = environ[i];	
			StringTokenizer st = new StringTokenizer(tempString);
			
			for(int j=0; j<sizeOfObjectsArray;j++)
			{
				objects[i-endOfLZs][j] = Integer.parseInt(st.nextToken());
			}			
		}		
			
		//Tokenizes the map header
		String mapString = environ[endOfObjs];
		
		StringTokenizer mapStringTokenizer = new StringTokenizer(mapString);
		mapWidth = Integer.parseInt(mapStringTokenizer.nextToken());
		mapLength = Integer.parseInt(mapStringTokenizer.nextToken());
		mapResolution = Integer.parseInt(mapStringTokenizer.nextToken());
		
		//Creates map height object of appropriate size
		mapHeights = new double[mapWidth][mapLength];
		
		for(int i=endHeightHeader; i<environ.length;i++) //Converts maps heights
		{
			String tempString = environ[i];
			StringTokenizer st = new StringTokenizer(tempString);
			
			for(int j=0; j<mapWidth; j++)
			{
				mapHeights[i-endHeightHeader][j] = Double.parseDouble(st.nextToken());
			}
			
		}
		
		//Creates new terrain map object
		tMap = new TerrainMap(mapResolution, mapHeights, textureID, landingSites, objects );
	
	}
	
	/**
	 * @return the planet's gravity
	 */
	public double getGravity()
	{
		return gravity;
		
	}
	
	/**
	 * @return the planet's name
	 */
	
	public String getPlanetName()
	{
		return planet_name;
		
	}
	
	//retuns picture file name

	public String getPlanetPictureFileName()
	{
		return planetPictureFileName;
		
	}
	/**
	 * @return the planet's radius
	 */
	public double getRadius()
	{
		return radius;
		
	}


	/**
	 * @return the mapLength
	 */
	public int getMapLength() {
		return mapLength;
	}

	/**
	 * @return the mapResolution
	 */
	public int getMapResolution() {
		return mapResolution;
	}

	/**
	 * @return the mapWidth
	 */
	public int getMapWidth() {
		return mapWidth;
	}

	/**
	 * @return the numberOfLandingSites
	 */
	public int getNumberOfLandingSites() {
		return numberOfLandingSites;
	}

	/**
	 * @return the numberOfObjects
	 */
	public int getNumberOfObjects() {
		return numberOfObjects;
	}
	
	/**
	 * @return the textureID
	 */
	public int getTextureID() {
		return textureID;
	}

	/**
	 * @return the objects
	 */
	public int[][] getObjects() {
		return objects;
	}
	/**
	 * @return the landingSites
	 */
	public int[][] getLandingSites() {
		return landingSites;
	}

	/**
	 * @return the mapHeights
	 */
	public double[][] getMapHeights() {
		return mapHeights;
	}
	

	/**
	 * @return the tMap
	 */
	public TerrainMap getTerrainMap() {
		return tMap;
	}
	
	//sets in game menu
	
	public void setMenu(Menu m)
	{
		inGameMenu = m;
	}
	
	//gets in game menu
	
	public Menu getMenu()
	{
		return inGameMenu;
	}

	public Object getServiceObject(IMyxName arg0) {
		// TODO Auto-generated method stub
		if (arg0.equals(msg)){
			return this;
		}
		return null;
	}
}
