package inf119.gamestate;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.StringTokenizer;

import edu.uci.isr.myx.fw.AbstractMyxSimpleBrick;
import edu.uci.isr.myx.fw.IMyxName;
import edu.uci.isr.myx.fw.MyxUtils;

import inf119.interfaces.GameDataIntf;

public class MyxGameData extends AbstractMyxSimpleBrick implements GameDataIntf {

	public static final IMyxName msg = MyxUtils.createName("GameData");
	
	//High Score
	//Username who got high score
	
	private String userName;
	
	private String[] stringGameData;
	private int[] gameOptions;

	private String defaultGameFileName;
	private int sizeOfGameDataFile;
	
	private ArrayList highScores;

	public MyxGameData() throws FileNotFoundException, IOException
	{
		
		defaultGameFileName = "saveGameDataTest.gdata";
		
		FileRead game_file = new FileRead();
		sizeOfGameDataFile = game_file.numberOfLinesInFile("saveGameDataTest.gdata");
		stringGameData = new String[sizeOfGameDataFile];
		stringGameData = game_file.readFile(defaultGameFileName, "game", sizeOfGameDataFile);

		//Load default options
		
		convertArrayIntoVars();

	}
	
	public void reloadGameData(String fileName) throws FileNotFoundException, IOException
	{
		FileRead game_file = new FileRead();
		stringGameData = new String[sizeOfGameDataFile];
		stringGameData = game_file.readFile(defaultGameFileName, "game", sizeOfGameDataFile);

		
		convertArrayIntoVars();
		
	}
	
	private void convertArrayIntoVars()
	{
		highScores = new ArrayList();
		
		for(int i =0; i<stringGameData.length; i++)
		{
			
			
			StringTokenizer  gameDataST =  new StringTokenizer(stringGameData[i]);
			String level = gameDataST.nextToken();
			String name = gameDataST.nextToken();
			double score = Double.parseDouble(gameDataST.nextToken());
			
			highScores.add(new HighScore(level, name, score));
		}
	}
	
//	default options set method
	
	
	public void setOption(int number, int value)
	{
		gameOptions[number] = value; 
	}

	
	public void setCurrentUserName(String name)
	{
		userName = name;
	}
	
	public String getCurrentUserName()
	{
		return userName;
	}
	
	public String getHighScoreUserName(String level)
	{
		for(int i=0; i<highScores.size(); i++)
		{
		
			if( ((HighScore)highScores.get(i)).sameLevel(level))
			{
				return ((HighScore)highScores.get(i)).getHighScoreUserName();
			}
		}
		
		highScores.add(new HighScore(level, null, 0));
		
		return null;
		
	}
	
	public double getHighScore(String level)
	{
		for(int i=0; i<highScores.size(); i++)
		{
			System.out.println("parameter: " +level + " H.S. name" + ((HighScore)highScores.get(i)).getLevelName());
			
			if( ((HighScore)highScores.get(i)).sameLevel(level))
			{
				return ((HighScore)highScores.get(i)).getHighScore();
			}
		}
		
		highScores.add(new HighScore(level, null, 0));
		
		return 0;
	}
	
	
	public boolean setHighScore(String level, double score)
	{
		boolean flag =doesLevelExist(level);
		
		if(flag==false)
		{
			highScores.add(new HighScore(level, getCurrentUserName(), score));
			return true;
		}	
		else
		{
			for(int i=0; i<highScores.size(); i++)
			{
				if( ((HighScore)highScores.get(i)).sameLevel(level) && ((HighScore)highScores.get(i)).isNewScoreLarger(score))
				{
					((HighScore)highScores.get(i)).setHighScore(score);
					((HighScore)highScores.get(i)).setHighScoreUserName(getCurrentUserName());
					return true;
					
				}
			}
			
			return false;
		}
		
	}
	
	private boolean doesLevelExist(String level)
	{
		for(int i=0; i<highScores.size(); i++)
		{
			if( ((HighScore)highScores.get(i)).sameLevel(level))
			{
				return true;
			}
			
		}
		
		return false;
	}

	public void saveData() throws IOException
	{
		FileWrite fw = new FileWrite(defaultGameFileName);
		for(int i=0; i<highScores.size(); i++)
		{
			fw.writeToken(((HighScore)highScores.get(i)).getLevelName());
			fw.writeToken(((HighScore)highScores.get(i)).getHighScoreUserName());
			fw.writeToken(((HighScore)highScores.get(i)).getHighScore());
			fw.endLine();
		}
		
		fw.close();
	}

	public Object getServiceObject(IMyxName arg0) {
		// TODO Auto-generated method stub
		if (arg0.equals(msg)){
			return this;
		}
		return null;
	}
}
