package inf119.gamestate;

import inf119.Constants;
import inf119.LanderModel;
import inf119.LocOri;
import inf119.TerrainMap;
import inf119.gamelogic.Menu;
import inf119.interfaces.*;

import java.io.FileNotFoundException;
import java.io.IOException;

import edu.uci.isr.myx.fw.AbstractMyxSimpleBrick;
import edu.uci.isr.myx.fw.IMyxName;
import edu.uci.isr.myx.fw.MyxUtils;

public class MyxGameStateConnector extends AbstractMyxSimpleBrick implements StateIntf{

	public static final IMyxName msg = MyxUtils.createName("state");
	public static final IMyxName pdmsg = MyxUtils.createName("PlayerData");
	public static final IMyxName gdmsg = MyxUtils.createName("GameData");
	public static final IMyxName edmsg = MyxUtils.createName("EnvironmentData");
	
	//***** Used Game State Connector *****//
	private EnvironmentDataIntf environData;
	private GameDataIntf currentGameData;
	private PlayerDataIntf currentPlayerData;

	private double currentFuelLevelInUnits;
	private int  sizeOfDataSets = 3;
	
	//***** Used Player Data *****//
	private double[] physicsShipVariables; //Array of variables passed to physics model
	private double[] physicsEnvironmentConstants; //Array of constants passed to physics model

	private int positionOfNonCovertedPhysicsNumbers; //Position of first number in physics ship variables array
													//that is not convereted.
	private int shipVariablesSize;  //Number of changing variables the physics model needs
	private int environmentConstantsSize; //Number of constants the physics model needs
	
	//***** Used for Menu. Sent to Player data *****//	
	private int shipInfoMenuSize;  //Amount of ship info that will be displayed in the menu
	private int scenarioInfoMenuSize; //Amount of scenario info that will be displayed in the menu
	private int planetInfoMenuSize; //Amount of planet info that will be displayed in the menu

	//***** User for HUD. Used in Game State Connector*****//
	private int speedPosition; //Position of the first speed variable in the physics arrary
	private int shipLocationPosition; //Position of the first location variable in the physics arrary
	private int shipAnglePosition; //Position of the first rotation variable in the physics arrary
	
	//***** Misc Files Loaded *****//
	private int locationOfFuelInFile; //Position in array in which the fuel number is stored (once ship file is parsed into a double array)
	
	
	//Sets variables and creates EnvironmentData, PlayerData, GameData
	public MyxGameStateConnector() throws FileNotFoundException, IOException
	{
	
		shipVariablesSize = 13;
		environmentConstantsSize = 8;
		
		positionOfNonCovertedPhysicsNumbers = 1;
		locationOfFuelInFile = 2;
		shipInfoMenuSize =4;
		scenarioInfoMenuSize= 7;
		planetInfoMenuSize= 2;
				
		physicsShipVariables = new double[shipVariablesSize];
		physicsEnvironmentConstants  = new double[environmentConstantsSize];
		
//		Location of variables in physics array
		speedPosition = 1;
		shipLocationPosition =4;
		shipAnglePosition = 7;
		
	}
	
	public void begin(){
		if (currentPlayerData==null){
			currentPlayerData = (PlayerDataIntf)MyxUtils.getFirstRequiredServiceObject(this, pdmsg);
			currentGameData = (GameDataIntf)MyxUtils.getFirstRequiredServiceObject(this, gdmsg);
			environData = (EnvironmentDataIntf)MyxUtils.getFirstRequiredServiceObject(this, edmsg);
		}
	}
	
	//Physics model methods
	public double[] getShipVariables()
	{
		/*length 13
		fuel_amount =0;
		x_speed =1;
		y_speed =2;
		z_speed =3;
		x_position=4;
		y_position=5;
		z_position=6;
		Yaw_Angle=7;
		Pitch_Angle = 8;
		Roll_Angle=9;
		Yaw_Omega =10;
		Roll_Omega =11;
		Pitch_Omega =12;
*/
		double[] scenarioStats = currentPlayerData.getScenarioNumbers();
		
		physicsShipVariables[0] =CurrentFuelLevelInUnits();
		
		for(int i= positionOfNonCovertedPhysicsNumbers; i<shipVariablesSize; i++)
		{
			physicsShipVariables[i] =scenarioStats[i];
		}
	
		return physicsShipVariables;
	}

	public  void setShipVariables(double[] newShipStats)
	{
		double percentOfFuelLeft = CurrentFuelLevelInPercent(newShipStats[0]);
		
		newShipStats[0] = percentOfFuelLeft;
		
		currentPlayerData.updateScenarioNumbers(newShipStats);
			
	}

	public double[] getEnvironmentConstants()
	{
		
/* file order
 * Mass =0
Ship radius =1
Max fuel =2
Thruster force=3 
Main Thruster Force=4
Ship height=5
Burn Rate=6
 */
		
/*order physics needs info
    planet_gravity =0;
    thruster_force=1;
    main_thruster=2;
    ship_mass=3;
    ship_radius=4;
    planet_radius=5; 
    ship_height=6; 
 */
		
		double[] shipStats = currentPlayerData.getShipNumbers();
		
		//ship stats array is 1 smaller than game file so everything is moved up one space
		
		physicsEnvironmentConstants[0] = environData.getGravity();
		physicsEnvironmentConstants[1] = shipStats[3];
		physicsEnvironmentConstants[2] = shipStats[4];
		physicsEnvironmentConstants[3] = shipStats[0];
		physicsEnvironmentConstants[4] = shipStats[1];
		physicsEnvironmentConstants[5] = environData.getRadius();
		physicsEnvironmentConstants[6] = shipStats[5];
		physicsEnvironmentConstants[7] = shipStats[6];
	    
		return physicsEnvironmentConstants;
	}
	
	
	//Difficulty settings
	public int getDifficulty()
	{
		return currentPlayerData.getDifficultyLevel();
	}
	
	public void setDifficulty(int level)
	{
		currentPlayerData.setDifficultyLevel(level);
	}
	
	
	//Load method for gui
	public void loadNewShip(String name) throws FileNotFoundException, IOException
	{
		if (currentPlayerData==null){
			begin();
		}
		currentPlayerData.loadShip(name, locationOfFuelInFile);	
	}
	
	public void loadNewScenario(String name) throws FileNotFoundException, IOException
	{
		currentPlayerData.loadScenario(name);

		if(getMenuState()== Constants.SCENARIOMODE || getMenuState()== Constants.LOADMODE)
		{
			String planetFileName = currentPlayerData.getPlanetFileName();	
			loadNewPlanet(planetFileName);	
		}
	}

	public void loadNewPlanet(String name) throws FileNotFoundException, IOException
	{
		environData.changePlanet(name);
		

	}

	public void loadNewGameData(String name) throws FileNotFoundException, IOException
	{
		currentGameData.reloadGameData(name);
		
	}

	
	
	//Name retrieval methods 
	public String getShipName()
	{
		return currentPlayerData.getShipName();
	}
	
	public String getPlanetName()
	{
		return environData.getPlanetName();
	}
	
	public String getPlanetFileName()
	{
		String planetFileName = currentPlayerData.getPlanetFileName();
		return planetFileName;
	}
	
	public String getScenarioName()
	{
		return currentPlayerData.getScenarioName();
	}
	
	
	//Menu info retrieval methods 
	public double[] getCurrentShipInfoForMenu()
	{
		double[] shipInfo = new double[shipInfoMenuSize];
		double[] shipInfoFullSet = currentPlayerData.getShipNumbers();
	
		for(int i=0; i<3; i++)
		{
			 shipInfo[i] = shipInfoFullSet[i];			
		}
		
		shipInfo[3] = shipInfoFullSet[5];
		/*
		 * mass
		 * Ship radius
		 * Max fuel
		 * Ship height
		 */
		
		return shipInfo;
	}
	
	public double[] getCurrentPlanetInfoForMenu()
	{
		double[] planetInfo = new double[planetInfoMenuSize];
		
		planetInfo[0] = environData.getGravity();
		planetInfo[1] = environData.getRadius();
		/*
		 * Gravity
		 * Radius
		 */
				
		return planetInfo;
	}
	
	public double[] getCurrentScenarioInfoForMenu()
	{
		double[] scenarioInfo = new double[scenarioInfoMenuSize];
		double[] scenarioInfoFullSet = currentPlayerData.getScenarioNumbers();
		
		for(int i=0; i<scenarioInfoMenuSize; i++)
		{
			scenarioInfo[i] = scenarioInfoFullSet[i];
		}
		
		/*
		 * Percent of Max Fuel
		 * X speed
		 * Y speed
		 * Z speed
		 * ship_x_position 
		 * ship_y_position 
		 * ship_z_position (this is their attitude) 
		 */
		return scenarioInfo;
	}
	
	public String getShipPictureFileName()
	{
		return currentPlayerData.getPictureFileName();
	}
	
	public String getPlanetPictureFileName()
	{
		return environData.getPlanetPictureFileName();
	}
	
	//High Score and Username methods
	
	public void setOption(int number, int value)
	{
		currentGameData.setOption(number, value);
	}

	
	public void setCurrentUserName(String name)
	{
		currentGameData.setCurrentUserName(name);
	}
	
	public String getCurrentUserName()
	{
		return currentGameData.getCurrentUserName();
	}

	public String getHighScoreUserName(String level)
	{
		return currentGameData.getHighScoreUserName(level);
	}
	
	public double getHighScore(String level)
	{
		return currentGameData.getHighScore(level);
		
	}
	
	public boolean setHighScore(String level, Double score)
	{
		return currentGameData.setHighScore(level, score);
	}

	public void saveHighScores() throws IOException
	{		
		currentGameData.saveData();
	}
	
	
	//Game/Menu State altering methods
	public int getGameState()
	{
		return currentPlayerData.getGameState();
	}
	
	public void updateGameState(int state)
	{
		currentPlayerData.updateGameState(state);
	}
	
	public int getMenuState()
	{
		return currentPlayerData.getMenuState();
	}
	
	public void updateMenuState(int state)
	{
		currentPlayerData.updateMenuState(state);
	}
	
	public void launchGameMenu()
	{
		currentPlayerData.updateGameState(Constants.MENUSTATE);
	}
	
	public void launchGame() throws FileNotFoundException, IOException
	{
		System.out.println("level " + currentPlayerData.getDifficultyLevel());
		currentPlayerData.setUpFuelNumbers();
		currentPlayerData.updateGameState(Constants.LOADMAPSTATE);
	}
		
	public void pauseGame()
	{
		currentPlayerData.updateGameState(Constants.PAUSESTATE);
	}
	
	public void endGame()
	{
		//currentPlayerData.updateGameState(Constants.ENDGAMESTATE);
	}
	
	
	//Fuel Methods
	public double getInitialFuelPercent()
	{
		return currentPlayerData.getInitialFuelPercent();
	}
	
	public double getCurrentFuelLevelPercent()
	{
		return currentPlayerData.getCurrentFuelLevelPercent();
	}
	
	
	//Camera Methods
	public LocOri getCameraInfo()
	{
		return currentPlayerData.getCameraInfo();
	}
	
	public  void setCameraInfo(LocOri camera)
	{
		
		currentPlayerData.updateCameraInfo(camera);
			
	}
	
	
//	Conversion methods
	private double CurrentFuelLevelInUnits()
	{
		
		 
		double maxFuel = currentPlayerData.getMaxFuelLevel();
		double percentLeft  = currentPlayerData.getCurrentFuelLevelPercent();
		double fuelInUnits;
		
		fuelInUnits = ((maxFuel * percentLeft)/100);
		currentFuelLevelInUnits = fuelInUnits;
		return currentFuelLevelInUnits;
		
	}

	private double CurrentFuelLevelInPercent(double currentLevelInUnits)
	{
			 
		double maxFuel = currentPlayerData.getMaxFuelLevel();
			
		double fuelInLeftInPercent;
		
		fuelInLeftInPercent = ((currentLevelInUnits/maxFuel)*100);
	
		return fuelInLeftInPercent;
		
	}

	
	/*****Game State Interpreter, HUD/Graphics Methods******/	

	
	public double[] getVelocity()
	{
		
		double[] tempShipVars = getShipVariables();
		double[] velocity = new double[sizeOfDataSets];
		int startOfArray =0;
		
		for(int i =speedPosition; i< (speedPosition+sizeOfDataSets); i++)
		{
			velocity[startOfArray] = tempShipVars[i];
			startOfArray++;
		}
		
		return velocity;
	}
	
	public double[] getShipPosition()
	{
		double[] tempShipVars = getShipVariables();
		double[] position = new double[sizeOfDataSets];
		int startOfArray =0;
		
		for(int i =shipLocationPosition; i< (shipLocationPosition+sizeOfDataSets); i++)
		{
			position[startOfArray] = tempShipVars[i];
			startOfArray++;
		}
		
		return position;		
	}

	public double[] getRotation()
	{				   
		
		double[] tempShipVars = getShipVariables();
		double[] rotation = new double[sizeOfDataSets];
		int startOfArray =0;
		
		for(int i =shipAnglePosition; i< (shipAnglePosition+sizeOfDataSets); i++)
		{
			rotation[startOfArray] = tempShipVars[i];
			startOfArray++;
		}
		
		return rotation;
	}

	public double getHUDFuelLevel()
	{
		double[] tempShipVars = getShipVariables();			
		return CurrentFuelLevelInPercent(tempShipVars[0]);		
	}

	public TerrainMap getTerrainMap()
	{
		return environData.getTerrainMap();
	}

	public LanderModel getLanderModel()
	{
		return currentPlayerData.getLanderModel();
	}
	
	//in game menu
	
	public void setMenu(Menu m)
	{
		environData.setMenu(m);
	}
	
	public Menu getMenu()
	{
		return environData.getMenu();
	}
	
	public void setThrusting(boolean thrust)
	{
		currentPlayerData.setThrusting(thrust);
	}
	
	public boolean getThrusting()
	{
		return currentPlayerData.getThrusting();
	}

	public Object getServiceObject(IMyxName arg0) {
		// TODO Auto-generated method stub
		if (arg0.equals(msg)){
			return this;
		}
		return null;
	}

}
