package inf119.gamestate;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.StringTokenizer;

import edu.uci.isr.myx.fw.AbstractMyxSimpleBrick;
import edu.uci.isr.myx.fw.IMyxName;
import edu.uci.isr.myx.fw.MyxUtils;

import inf119.Constants;
import inf119.LanderModel;
import inf119.LocOri;
import inf119.interfaces.PlayerDataIntf;

public class MyxPlayerData extends AbstractMyxSimpleBrick implements PlayerDataIntf {
	
	public static final IMyxName msg = MyxUtils.createName("PlayerData");
	
	//Ship variables
	private int number_of_ship_vars; //Size of array that contains the ship file
	private int number_of_ship_numbers; //Size of array of parsed information from ship file
	private String[] ship; //String array representation of ship file
	private double[] shipNumbers;//Double array representation of ship file
	private int numberOfNonNumberVarsInShipFile; //Number of lines in ship file not parsed
	private int locationOfFuelInFile; //Position in array in which the fuel number is stored (once ship file is parsed into a double array)
	private String shipName; //Ship's name
	private String shipFileName; //Ship's file name
	private String shipModelFileName; //Ship model file name that corresponds to this ship
	private String shipPictureFilename; //Ship picture file name that corresponds to this ship
	private String defaultShipFileName;  //Name of default ship that is loaded on creation of class
	private LanderModel model;
	private boolean isThrusting;  //not written to file; used only for graphics
	 
	//Scenario Variables
	private int number_of_scenario_vars; //Size of array that contains the scenario file
	private int number_of_scenario_numbers; //Size of array of parsed information from scenario file
	private String scenarioName; //Name of scenario
	private String[] scenario;//String array representation of scenario file
	private double[] scenarioNumbers;//Double array representation of scenario file	
	private int numberOfNonNumberVarsInScenarioFile; //Number of lines in scenario file not parsed
													//The 3rd line technically is parsed but not included because it is not
													//passed to the physics model
	private int number_of_Number_VarsInScenarioFile; //Size of string array from scenario file that get parsed
													 //includes physics numbers and camera position and rotation
	private String defaultScenarioFileName;//Name of default scenario that is loaded on creation of class
	private double[] parseArray;  //Contains parsed scenario file data before the data is put into the camera and scenario arrays
	
	
	//Camera variables
	private LocOri camera;  //Camera object
	private int cameraSize;  //Number of variables needed to create camera object
	private double[] initialCameraSettings; //Camera settings read from file
	
	//Fuel variables
	private double maxFuelLevel; //Max fuel level of ship
	private double initialFuelPercent; // % of fuel ship starts with
	private double currentFuelLevelPercent; //Current % of fuel in ship
		
	//Misc
	private int gameState = Constants.EASYDIFFICULTY; //Contains state of game
	private int menuState = Constants.MENUSTATE;	//Contains state of menu
	private String planetFileName; //Planet's name
	private int difficultyLevel =Constants.EASYDIFFICULTY; //Current difficulty setting
	
	
	
	//	Sets variables and loads default ship and scenario
	public MyxPlayerData() throws FileNotFoundException, IOException
	{
		
		number_of_ship_vars = 10;
		numberOfNonNumberVarsInShipFile = 3;
		number_of_ship_numbers = number_of_ship_vars - 3; //Minus 3 for the filenames and such
		
		number_of_scenario_vars =23;
		cameraSize= 6;
		numberOfNonNumberVarsInScenarioFile = 4;
		
		//Number of variables that will be passed to physics model = Total number of lines in scenario file - 
		//number of non number lines scenario file - number of camera vars
		number_of_scenario_numbers = number_of_scenario_vars - 4 -cameraSize;
		
		//Number of variables in scenario file that need to be parsed.
		number_of_Number_VarsInScenarioFile = number_of_scenario_vars - numberOfNonNumberVarsInScenarioFile;
		
		/*
		System.out.println("Scenario file length " +number_of_scenario_vars);
		System.out.println("Number of non number Vars In Scenario File " +numOfNonNumberVarsInScenarioFile);
		System.out.println("Camera size " +cameraSize);
		System.out.println("Should be number of vars david uses (total - front end - camera) " + number_of_scenario_numbers);
		System.out.println("should be the number of vars need to be parsed (total-front end) " +number_of_Number_VarsInScenarioFile);
		*/
		
		defaultShipFileName = "defaultship.ship";
		defaultScenarioFileName = "defaultscenario.scenario";
		
		locationOfFuelInFile = 2;
		loadShip(defaultShipFileName, locationOfFuelInFile);
		loadScenario(defaultScenarioFileName);
		
	}
	
	//Loads a new ship
	public void loadShip(String filename, int localOfFuelInFile) throws FileNotFoundException, IOException
	{
		//Reads in ship file
		FileRead ship_file = new FileRead();
		ship = new String[number_of_ship_vars];		
		ship = ship_file.readFile(filename, "shipfile", number_of_ship_vars); 
		
		//Creates arrays to hold data
		shipNumbers = new double[(number_of_ship_numbers)]; 
		//number_of_ship_numbers is one less so it only does numbers and not the ship name
		
		convertArrayIntoVars("ship"); //Coverts string array into usable vars
		createLanderModel(shipModelFileName);		
		maxFuelLevel= shipNumbers[localOfFuelInFile]; //Sets the max fuel level for particular ship
		
	}
	
	//Loads a new scenario
	public void loadScenario(String filename) throws FileNotFoundException, IOException
	{
		//Reads in scenario file		
		FileRead scenario_file = new FileRead();
		scenario = new String[number_of_scenario_vars];
		scenario = scenario_file.readFile(filename, "scenario", number_of_scenario_vars); 
		
		//Creates arrays to hold data
		scenarioNumbers = new double[(number_of_scenario_numbers)];
		parseArray = new double[number_of_Number_VarsInScenarioFile];
		initialCameraSettings = new double[cameraSize];
		
		convertArrayIntoVars("scenario"); //Coverts string array into usable vars
		
		createCamera(initialCameraSettings); //Creates new camera object based on file
		
		setUpFuelNumbers(); //Sets up fuel numbers.
		
			
	}
	
	//Converts string array into proper values
	private void convertArrayIntoVars(String type) throws FileNotFoundException, IOException
	{
		
		if(type.equals("ship"))
		{
			shipName = ship[0]; //Gets ship name
			shipModelFileName = ship[1];
			shipPictureFilename = ship[2];
			
			//Parses ship values
			for(int i =numberOfNonNumberVarsInShipFile; i<number_of_ship_vars; i++)
			{	
				
				shipNumbers[(i-numberOfNonNumberVarsInShipFile)] = Double.parseDouble(ship[i]);
			}				
		}
		else
		{
			//If in scenario or load mode it must caputre the ship and planet names
			//from the file
			if(getMenuState()== Constants.SCENARIOMODE || getMenuState()== Constants.LOADMODE)
			{
				scenarioName = scenario[0];
				shipFileName = scenario[1];
				planetFileName = scenario[2];
				difficultyLevel = Integer.parseInt(scenario[3]); 
				loadShip(shipFileName, locationOfFuelInFile);						
			}
			
			//Parses scenario values
			for(int i =numberOfNonNumberVarsInScenarioFile; i<number_of_scenario_vars; i++)
			{
				parseArray[i-numberOfNonNumberVarsInScenarioFile] = Double.parseDouble(scenario[i]);
			}
			
			//Decides if value should go into scenario number array or initial camera settings
			for(int i = 0; i < parseArray.length;i++)
			{
				if(i<number_of_scenario_numbers)
				{
					scenarioNumbers[i] = parseArray[i];
				}
				else
				{
					initialCameraSettings[i-number_of_scenario_numbers] = parseArray[i];
				}
			}	
		}				
	}
	
	//Takes in scenario numbers and updates the current ones
	//Uses a for loop because game state connector does
	public void updateScenarioNumbers(double[] currentStats)
	{
		for(int i =0; i < currentStats.length; i++)
		{
			scenarioNumbers[i] =currentStats[i];		
		}
		currentFuelLevelPercent = scenarioNumbers[0]; 
	}
	
	//Sets up the starting fuel amount based up difficulty
	public void setUpFuelNumbers()
	{
		if(getMenuState()==Constants.FREEMODE)
		{
			currentFuelLevelPercent = scenarioNumbers[0];
			initialFuelPercent = scenarioNumbers[0];			
			
		}
		else
		{
			if(difficultyLevel==Constants.EASYDIFFICULTY)
			{
				currentFuelLevelPercent = scenarioNumbers[0];
				initialFuelPercent = scenarioNumbers[0];
			}
			else if(difficultyLevel==Constants.NORMALDIFFICULTY)
			{
				currentFuelLevelPercent = (Constants.PERCENTAGEOFUELONNORMAL *  scenarioNumbers[0]);
				initialFuelPercent = (Constants.PERCENTAGEOFUELONNORMAL *  scenarioNumbers[0]);			
			}
			else if(difficultyLevel==Constants.HARDDIFFICULTY)
			{
				System.out.println("HArd");
				currentFuelLevelPercent = (Constants.PERCENTAGEOFUELONHARD *  scenarioNumbers[0]);
				initialFuelPercent = (Constants.PERCENTAGEOFUELONHARD *  scenarioNumbers[0]);					
			}
			else
			{
				currentFuelLevelPercent = scenarioNumbers[0];
				initialFuelPercent = scenarioNumbers[0];		
			}	
		}
	}
	
	
	
	//Returns ship variables	
	public double[] getShipNumbers()
	{
		return shipNumbers;
	}	
	
	//Returns scenario numbers
	public double[] getScenarioNumbers()
	{
		return scenarioNumbers;
	}
	
	
	/*****Game/Menu Methods******/
	
	//Sets game state
	public int getGameState()
	{
		return gameState;
		
	}
	
	//Returns game state
	public void updateGameState(int state)
	{
		gameState =state;
	}
	
	//Returns menu state
	public int getMenuState()
	{
		return menuState;
		
	}
	
	//Sets menu state
	public void updateMenuState(int state)
	{
		menuState =state;
	}
	

	/*****Difficulty Methods******/
	
	//Returns difficult level
	public int getDifficultyLevel()
	{
		return difficultyLevel;
	}
	
	//Set difficulty level
	public void setDifficultyLevel(int level)
	{
		difficultyLevel = level;
		setUpFuelNumbers();
	}
	
	
	/*****Fuel Methods******/
	//Returns initial fuel %
	public double getInitialFuelPercent()
	{
		return initialFuelPercent;
	}
	
	//Returns current fuel %
	public double getCurrentFuelLevelPercent()
	{
		return currentFuelLevelPercent;
	}
	
	//Returns max fuel level
	public double getMaxFuelLevel()
	{

		return maxFuelLevel;
	}
	
	/*****Camera Methods******/
	public void updateCameraInfo(LocOri cam) //Updates camera object
	{	
		camera = cam;		
	}
	
	public LocOri getCameraInfo()//Returns camera object
	{
		return camera;
	}

	private void createCamera(double[] currentCameraInfo)//Creates camera object
	{	
		camera = new LocOri(currentCameraInfo);		
	}
	
	
	/*****Lander Model Methods******/	
	
	//Creates Lander Model object 
	public void createLanderModel(String filename) throws FileNotFoundException, IOException
	{
		
		String[] lander;
		int sizeOfLanderModelFile;
		
		String landerName;
		String landerDescription;
		int texFaces;
		double resolution;
		int[] textureIDs;
		double height;
		double[] thrustLocation; 
		double thrustRadius;
		double[] centerOfMass;
		double[][][] vertices;
		
		int readlinePostion =0;
		
		
		FileRead lander_model = new FileRead();
		sizeOfLanderModelFile = lander_model.numberOfLinesInFile(filename);
		
		lander = new String[sizeOfLanderModelFile];		
		lander = lander_model.readFile(filename, "landermodelfile", sizeOfLanderModelFile);
		
		
		
		landerName = lander[readlinePostion];
		readlinePostion++;
		landerDescription = lander[readlinePostion];
		readlinePostion++;
		resolution = Double.parseDouble(lander[readlinePostion]);
		readlinePostion++;
		texFaces = Integer.parseInt(lander[readlinePostion]);
		readlinePostion++;
		
		StringTokenizer textureST= new StringTokenizer(lander[readlinePostion]);
		textureIDs = new int[textureST.countTokens()];
		for(int i=0; i< textureIDs.length; i++)
		{
			textureIDs[i] =  Integer.parseInt(textureST.nextToken());
		}
		readlinePostion++;
		
		height = Double.parseDouble(lander[readlinePostion]);
		readlinePostion++;
		
		
		StringTokenizer  thrustLocationST =  new StringTokenizer(lander[readlinePostion]);
		thrustLocation = new double[thrustLocationST.countTokens()];
		for(int i=0; i< thrustLocation.length; i++)
		{
			thrustLocation[i] =  Double.parseDouble(thrustLocationST.nextToken());
		}
		readlinePostion++;

		
		thrustRadius = Double.parseDouble(lander[readlinePostion]);
		readlinePostion++;

		
		StringTokenizer centerOfMassST =  new StringTokenizer(lander[readlinePostion]);
		centerOfMass = new double[centerOfMassST.countTokens()];
		for(int i=0; i< centerOfMass.length; i++)
		{
			centerOfMass[i] =  Double.parseDouble(centerOfMassST.nextToken());
		}
		readlinePostion++;
		

		int depth = (sizeOfLanderModelFile-readlinePostion)/4;
		                            
		vertices = new double[depth][4][3];
	
		for(int face=0; face<vertices.length ; face++)
		{
			
			for(int vertex=0; vertex< vertices[0].length ; vertex++)
			{
				StringTokenizer verticesST =  new StringTokenizer(lander[readlinePostion]);

				for(int coord=0; coord< vertices[0][0].length ; coord++)
				{
					vertices[face][vertex][coord] = Double.parseDouble(verticesST.nextToken());
				}
				
				readlinePostion++;
			}
		}
		
		model  = new LanderModel(resolution, textureIDs, vertices, texFaces, thrustLocation, thrustRadius, height, centerOfMass);
	}
	
	//Returns Lander Model object
	public LanderModel getLanderModel()
	{
		return model;
	}
	
	
	/*****Misc Methods******/
	
	//Returns planet file name
	public String getPlanetFileName()
	{
		return planetFileName;
	}
	
	//Returns ship name 
	public String getShipName()
	{
		return shipName;
	}
	
	//Returns the picture filename associated with this ship file
	public String getPictureFileName()
	{
		return shipPictureFilename;
	}
	
	//Returns scenario name 
	public String getScenarioName()
	{
		return scenarioName;
	}
	
	//sets whether or not main thruster is in use
	public void setThrusting(boolean thrust)
	{
		isThrusting = thrust;
	}
	
	//returns true if main thruster is engaged
	public boolean getThrusting()
	{
		return isThrusting;
	}

	public Object getServiceObject(IMyxName arg0) {
		// TODO Auto-generated method stub
		if (arg0.equals(msg)){
			return this;
		}
		return null;
	}
}
