package inf119.graphics;

import inf119.LanderModel;
import inf119.LocOri;
import inf119.TerrainMap;
import inf119.gamelogic.Menu;
import inf119.interfaces.*;

import java.awt.Font;

import javax.media.opengl.DebugGL;
import javax.media.opengl.GL;
import javax.media.opengl.GLAutoDrawable;
import javax.media.opengl.GLDrawable;
import javax.media.opengl.GLEventListener;
import javax.media.opengl.glu.GLU;

import com.sun.opengl.util.j2d.TextRenderer;

import edu.uci.isr.myx.fw.AbstractMyxSimpleBrick;
import edu.uci.isr.myx.fw.IMyxName;
import edu.uci.isr.myx.fw.MyxUtils;

public class MyxGraphicsMaster extends AbstractMyxSimpleBrick  implements GLEventListener, GraphicsIntf {

	public static final IMyxName msg = MyxUtils.createName("graphics");
	public static final IMyxName statemsg = MyxUtils.createName("state");
	public static final IMyxName txtmsg = MyxUtils.createName("Texture");
	public static final IMyxName lamsg = MyxUtils.createName("LanderArtist");
	public static final IMyxName samsg = MyxUtils.createName("SkyboxArtist");
	public static final IMyxName tamsg = MyxUtils.createName("TerrainArtist");
	public static final IMyxName eamsg = MyxUtils.createName("EffectsArtist");
	public static final IMyxName mamsg = MyxUtils.createName("MenuArtist");
	public static final IMyxName hamsg = MyxUtils.createName("HudArtist");
	
	public StateIntf gsc;
	
	private final double PERSPECTIVE_ANGLE = 45, DEFAULT_TERRAIN_SIZE = 5,
	NEAR_PLANE = 0.1, FAR_PLANE = 5000, LANDER_NORMAL_SIZE = 0.1;

	private LanderArtistIntf 	la;
	private SkyboxArtistIntf 	sa;
	private TerrainArtistIntf 	ta;
	private EffectsArtistIntf 	ea;
	private MenuArtistIntf		ma;
	private HudArtistIntf		ha;

	private TextureIntf 	tm;	
	private GL				gl;
	private GLU 			glu;
	private GLDrawable		glDrawable;
	private TextRenderer	texRen;
	private int 			width, height;
	private static boolean	isThrusting = false, drawMenu = false, hasCrashed = false, isFullscreen = false, useGuide = true;
	private int				counter = 0, increment = 1;	
	public static final int	COUNTER_MAX = 360;
	private static double	metersPerUnit = 100;


	//Camera position in 3D space
	private double[]	camPosition = {0,20,0};
	//Rotations for the camera about the (x-axis, y-axis, z-axis)
	private double[]	camRotation = {0,20,0};
	//Translation for the lander
	private double[]	landerPos = {0,20,0};
	//Rotations for the lander about the (x-axis, y-axis, z-axis)
	private double[]	landerRotation = {0,0,0};
	//size of the terrain 
	private double 		terrainSize = DEFAULT_TERRAIN_SIZE;
	//altitude of the lander
	private double		landerAltitude = 0.0;
	//velocity of the lander
	private double		landerVelocity = 0.0;
	//fuel level of the lander
	private double		landerFuel = 100.0;

	private Font myFont = new Font("SansSerif", Font.PLAIN, 14);

	public static int widthOfString(Font font, String input)
	//Calculates the approximate width of the string in pixels.
	{
		return (font.getSize() * input.length());
	}

	public static void setMetersPerUnit(double mpu)
	{
		metersPerUnit = mpu;
	}

	public static double metersToUnits(double meters)
	{
		return (meters/metersPerUnit);
	}
	
	public static double unitsToMeters(double units)
	{
		return (units*metersPerUnit);
	}

	public void toggleCrashed()
	{
		hasCrashed = !hasCrashed;
	}

	public void setThrusting(boolean thrust)
	{
		isThrusting = thrust;
	}

	public Font getFont()
	{
		return myFont;
	}

	public MyxGraphicsMaster()
	{
		this.width = 800;
		this.height = 600;
	
		this.glu = new GLU();
		texRen = new TextRenderer(myFont);
		//tm = new TextureManager();
		
	}
	
	public void begin(){
		gsc = (StateIntf)MyxUtils.getFirstRequiredServiceObject(this, statemsg);
		tm = (TextureIntf)MyxUtils.getFirstRequiredServiceObject(this, txtmsg);
		la = (LanderArtistIntf)MyxUtils.getFirstRequiredServiceObject(this, lamsg);
		ta = (TerrainArtistIntf)MyxUtils.getFirstRequiredServiceObject(this, tamsg);
		ta.setTR(texRen);
		sa = (SkyboxArtistIntf)MyxUtils.getFirstRequiredServiceObject(this, samsg);
		ea = (EffectsArtistIntf)MyxUtils.getFirstRequiredServiceObject(this, eamsg);
		ha = (HudArtistIntf)MyxUtils.getFirstRequiredServiceObject(this, hamsg);
		ha.setTR(texRen);
		ma = (MenuArtistIntf)MyxUtils.getFirstRequiredServiceObject(this, mamsg);
		ma.setTR(texRen);
	}

	public void setMap()
	{
		TerrainMap map = gsc.getTerrainMap();
		ta.setTerrainMap(map);
		int skyboxLength = 4*(Math.max(map.heights.length, map.heights[0].length));		
		//In meters, the skybox length = 4*(# cells in longest edge of terrain) * (meters per terrain cell)
		sa.setSkyboxLength(map.resolution * skyboxLength);
	}

	public void setMenu(Menu m)
	{
		drawMenu = (m != null);
		ma.setMenu(m);
	}

	public void setLanderModel(LanderModel lm)
	{
		la.setLanderModel(lm);
		ea.setThrust(lm.thrustRad, lm.thrustLoc[1]);
	}

	public void setCamera(LocOri cam)
	{
		double[] vals = cam.toSingleArray();
		for (int i = 0; i < 3; i++){
		camPosition[i] = metersToUnits(vals[i]);
		camRotation[i] = vals[i+3];
		}		
	}
	//System.out.print("CAM: " + camPosition[0] + ", " + camPosition[1] + ", " + camPosition[2] + "\n");


	public void setLanderPosition(double[] position)
	{
		if (position.length != 3)
			return;
		for (int i = 0; i < 3; i++){
			landerPos[i] = position[i];
		}
	}
	
	public void setLanderRotation(double[] rotation)
	/*
	 * rotation is {xA, yA, zA} where xA is the rotation about the X axis in radians, etc.
	 * stores the rotations in degrees
	 */
	{
		if (rotation.length != 3)
			return;
		for (int i = 0; i < 3; i++){
			landerRotation[i] = rotation[i] * 180 / Math.PI;
		}
	}

	public void setLander(double[] position, double[] rotation)
	/*
	 * rotation is {xA, yA, zA} where xA is the rotation about the X axis in radians, etc.
	 * stores the rotations in degrees
	 */	
	{
		if (position.length != 3 || rotation.length != 3)
			return;
		for (int i = 0; i < 3; i++){
			landerPos[i] = position[i];
			landerRotation[i] = rotation[i] * 180 / Math.PI;
		}
	}

	public void setLanderVelocity(double velocity)
	{
		landerVelocity = Math.abs(velocity);
	}

	public void setLanderAltitude(double altitude)
	{
		if( altitude < 0 )
			landerAltitude = 0;
		else landerAltitude = altitude;
	}

	public void setLanderFuel(double fuelLevel)
	{
		if( fuelLevel < 0 )
			landerFuel = 0;
		else landerFuel = fuelLevel;
	}

	private void placeCamera(double[] position, double[] rotation)
	{
		//System.out.println("placeCamera: " + position[0] + "," + position[1] + "," + position[2]);
		gl.glLoadIdentity();
		//Rotate Camera along coordinate axes
			
		gl.glRotated(rotation[0], 1.0, 0.0, 0.0);
		gl.glRotated(rotation[1], 0.0, 1.0, 0.0);
		gl.glRotated(rotation[2], 0.0, 0.0, 1.0);
	
		//Translate Camera according to position
		gl.glTranslated(-position[0], -position[1], -position[2]);
	}

	public LocOri getCamera()
	/* 
	 * Note: this function is only for use with the Graphics Harness demo
	 * 
	 */
	{
		LocOri ret = new LocOri(camPosition, camRotation);
		return ret;
	}	

	public void toggleFullscreen()
	{
		isFullscreen = !isFullscreen;

	}

	public void display(GLAutoDrawable arg0) {
		gl.glClear(GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT);	

		if (!drawMenu){
			counter = (counter + increment) % COUNTER_MAX;
		
			placeCamera(camPosition, camRotation);
			sa.drawSkybox();
			sa.drawEarth();
			ta.drawTerrain(terrainSize,0);
			ta.drawLandingZones(counter);
			la.drawLander(landerPos, landerRotation);
			ea.drawEffects(landerPos, landerRotation, counter, isThrusting, hasCrashed, la.getThrustLoc(), la.getResolution(), la.getCentMass(), useGuide);
			ha.drawHUD(glDrawable.getWidth(), glDrawable.getHeight(), landerAltitude, landerVelocity, landerFuel, landerRotation[0], landerRotation[1], landerRotation[2]);
		
		}
		if (drawMenu)
			ma.DrawMenu(glDrawable.getWidth(), glDrawable.getHeight());
	}

	public void init(GLAutoDrawable drawable) {
		//Initialize the GL related objects
		this.gl = drawable.getGL();
		this.glDrawable = drawable;
	
		la.setGL(gl);
		ta.setGL(gl);
		sa.setGL(gl);
		ea.setGL(gl);
		ha.setGL(gl);
		ma.setGL(gl);
	
		drawable.setGL( new DebugGL(drawable.getGL() ));
		System.out.println("Init GL is " + gl.getClass().getName());
		tm.init(gl, glu);	//Initialize the texture manager
	
		gl.glMatrixMode(GL.GL_PROJECTION);	//Setup the projection matrices
		gl.glLoadIdentity();
		glu.gluPerspective(PERSPECTIVE_ANGLE, (width)/(double)height, NEAR_PLANE, FAR_PLANE);

		gl.glMatrixMode(GL.GL_MODELVIEW);	//Switch back to model-view matrices
		gl.glEnable(GL.GL_CULL_FACE); // Enable Backface culling
		gl.glEnable( GL.GL_DEPTH_TEST );
		gl.glDepthFunc(GL.GL_LEQUAL);
	}

	public int getWidth()
	{
		//System.out.println("first goes here");
		return width;
	}

	public int getHeight()
	{
		//System.out.println("second goes here");
		return height;
	}

	public void reshape(GLAutoDrawable drawable, int x, int y, int width, int height) {
		this.width = width;
		this.height = height;
		gl.glMatrixMode(GL.GL_PROJECTION);	//Setup the projection matrices
		gl.glLoadIdentity();
		glu.gluPerspective(PERSPECTIVE_ANGLE, (width)/(double)height, NEAR_PLANE, FAR_PLANE);
		gl.glMatrixMode(GL.GL_MODELVIEW);	//Switch back to model-view matrices
	}

	public void displayChanged(GLAutoDrawable arg0, boolean arg1, boolean arg2) {
	}
	
	public Object getServiceObject(IMyxName arg0) {
		// TODO Auto-generated method stub
		if (arg0.equals(msg)){
			return this;
		}
		return null;
	}

}
