package inf119.graphics;

import javax.media.opengl.GL;

import com.sun.opengl.util.j2d.TextRenderer;

import edu.uci.isr.myx.fw.AbstractMyxSimpleBrick;
import edu.uci.isr.myx.fw.IMyxName;
import edu.uci.isr.myx.fw.MyxUtils;

import inf119.LandingZone;
import inf119.TerrainMap;
import inf119.interfaces.TerrainArtistIntf;
import inf119.interfaces.TextureIntf;

public class MyxTerrainArtist extends AbstractMyxSimpleBrick implements TerrainArtistIntf {

	public static final IMyxName msg = MyxUtils.createName("TerrainArtist");
	public static final IMyxName txtmsg = MyxUtils.createName("Texture");
	
	GL gl;
	TextureIntf tm;
	TerrainMap map;
	TextRenderer texRen;
	final double NUDGE = 1; //Used for layered flat objects so that they are drawn
							//One on top of the other	
	
	final static int 	RING_SIDES = 12, SUB_RINGS = 12;
	
	final static double SQUARE_V[][][] =
	{
	  {
	    {-1.0, -1.0, -1.0},
	    {-1.0, -1.0, 1.0},
	    {1.0, -1.0, 1.0},
	    {1.0, -1.0, -1.0}
	  }	 
	};

	final static double DEFAULT_RES = 100;
	
	final static double DEFAULT_HS[][] = 
	{ 
		{0,0,0,0,0,0,0},
		{0,0,0,0,0,0,0},
		{0,0,0.5,0.5,0.5,0,0},
		{0,0,0.5,0.5,0.5,0,0},
		{0,0,0.5,0.5,0.5,0,0},
		{0,0,0,0,0,0,0},
		{0,0,0,0,0,0,0}
	};
	
	public MyxTerrainArtist()
	{
		//This is default map data
		//The terrain map should be updated from the state
		int[][] zones = {{0,0,50,500}}; 
		int[][] obj = {{1,2,1}};
		map = new TerrainMap(DEFAULT_RES, DEFAULT_HS, 1, zones, obj);
	}

	public void setGL(GL g)
	{
		gl = g;
	}
	
	public void setTR(TextRenderer tr){
		texRen = tr;
	}

	public void setTerrainMap(TerrainMap tm)
	{
		map = tm;
	}
	
	public void drawTerrain(double len, double alt)
	{
		if (map.heights == null){
			System.err.println("Error: attempting to draw a null terrain");
			return;
		}
		int cellsX = map.heights.length,  //size along x axis in cells
			cellsZ = map.heights[0].length;//size along z axis in cells
		
		gl.glPolygonMode(GL.GL_FRONT_AND_BACK, GL.GL_FILL);
    	gl.glEnable( GL.GL_TEXTURE_2D );
    	gl.glBindTexture( GL.GL_TEXTURE_2D, tm.getTexID(map.textureID) );
    	gl.glTexEnvf(GL.GL_TEXTURE_ENV, GL.GL_TEXTURE_ENV_MODE, GL.GL_DECAL);

    	gl.glPushMatrix();
    	
	    
	    double res = map.resolution, units = MyxGraphicsMaster.metersToUnits(res);
	    gl.glTranslated((-(cellsX-1)*(units))/2, 0, (-(cellsZ-1)*(units))/2);
	    gl.glScaled(MyxGraphicsMaster.metersToUnits(res),
	    		MyxGraphicsMaster.metersToUnits(res),
	    		MyxGraphicsMaster.metersToUnits(res));
     
    	gl.glColor3d(0.5, 0.5, 0.5);
    	
    	
		for (int i = 0; i < cellsX - 1; i++){
			for (int j = 0; j < (cellsZ - 1); j++){
				//Draw the upper left triangle
				gl.glBegin(GL.GL_TRIANGLES);
				gl.glTexCoord2d( 0, 0 );
				gl.glVertex3d(i, map.heights[i][j], j);
				gl.glTexCoord2d( 1, 1 );
				gl.glVertex3d(i+1, map.heights[i+1][j+1], j+1);
				gl.glTexCoord2d( 1, 0 );
				gl.glVertex3d(i+1, map.heights[i+1][j], j);
				gl.glEnd();

				//Draw the lower right triangle
				gl.glBegin(GL.GL_TRIANGLES);
				gl.glTexCoord2d( 0, 0 );
				gl.glVertex3d(i, map.heights[i][j], j);
				gl.glTexCoord2d( 0, 1 );
				gl.glVertex3d(i, map.heights[i][j+1], j+1);
				gl.glTexCoord2d( 1, 1 );
				gl.glVertex3d(i+1, map.heights[i+1][j+1], j+1);
				gl.glEnd();
				
			}
		}
		gl.glPopMatrix();
		gl.glEnable(GL.GL_CULL_FACE); // Enable Backface culling
	}
	
	/*
	 * This method draws the landing sites
	 * sites[n][4] where sites[n] = {X,Z,size, score} for the nth site
	 */
	
	public void drawLandingZones(int counter)
	{
		if (map == null)
			return;
		LandingZone[] zones = map.getLandingZones();
		for (int i = 0; i < zones.length; i++){
			drawLandingZone(zones[i], counter);
		}
	}
	
	private double zoneHeight(double xM, double zM, double rM)
	/* 
	 * Given coordinates in meters
	 * Returns the maximum height of the four corners of the landing zone in meters
	 * Assumes that the map makers did not put spikes within the zone
	 * For best results: only place zones on even ground  
	 */
	
	{
		double x = MyxGraphicsMaster.metersToUnits(xM),
				z = MyxGraphicsMaster.metersToUnits(zM),
				radius = Math.round(MyxGraphicsMaster.metersToUnits(rM));
		
		double baseX = x + map.heights.length/2, baseZ = z + map.heights.length/2;
		double maxHeight = map.heightAt(baseX - radius, baseZ-radius);
		maxHeight = Math.max(maxHeight, map.heightAt(baseX+radius, baseZ-radius));
		maxHeight = Math.max(maxHeight, map.heightAt(baseX-radius, baseZ+radius));
		maxHeight = Math.max(maxHeight, map.heightAt(baseX+radius, baseZ+radius));
		return (maxHeight*map.resolution) + NUDGE;
	}
	
	/*
	 * This method draws the landing site
	 * 
	 */
	private void drawLandingZone(LandingZone zone, int counter)
	{
		gl.glPolygonMode(GL.GL_FRONT_AND_BACK, GL.GL_FILL);
	    gl.glEnable( GL.GL_TEXTURE_2D );
	    gl.glBindTexture( GL.GL_TEXTURE_2D, tm.getTexID(map.textureID) );
	    gl.glTexEnvf(GL.GL_TEXTURE_ENV, GL.GL_TEXTURE_ENV_MODE, GL.GL_MODULATE);
	    
	    gl.glPushMatrix();
	    
	    double zoneSize = zone.radius * 2,
	    	zoneUnits = MyxGraphicsMaster.metersToUnits(zoneSize);
	    double height = zoneHeight(zone.x,zone.z,zoneSize);
	    gl.glTranslated(MyxGraphicsMaster.metersToUnits(zone.x),
	    		MyxGraphicsMaster.metersToUnits(height),
	    		MyxGraphicsMaster.metersToUnits(zone.z));   
	    gl.glScaled(MyxGraphicsMaster.metersToUnits(zoneSize), 1, MyxGraphicsMaster.metersToUnits(zoneSize));
	    
	    
	    gl.glBegin(GL.GL_POLYGON);
	      gl.glColor3d(0.7, 0.7, 0.7);
	      
	      gl.glTexCoord2d( 0, 0 );
	      gl.glVertex3d(SQUARE_V[0][0][0], 0, SQUARE_V[0][0][2]);
	      gl.glTexCoord2d( 0, zoneUnits );
	      gl.glVertex3d(SQUARE_V[0][1][0], 0, SQUARE_V[0][1][2]);
	      gl.glTexCoord2d( zoneUnits, zoneUnits );
	      gl.glVertex3d(SQUARE_V[0][2][0], 0, SQUARE_V[0][2][2]);
	   	  gl.glTexCoord2d( zoneUnits, 0 );
	   	  gl.glVertex3d(SQUARE_V[0][3][0], 0, SQUARE_V[0][3][2]);
	    gl.glEnd();
	      
	    gl.glPopMatrix();
	    
	    
	    /*
	     * Draw landing site point value 
	     * TODO: make sure gl state bits are restored
	     */
	    double xU = MyxGraphicsMaster.metersToUnits(zone.x),
	    	zU = MyxGraphicsMaster.metersToUnits(zone.z),
	    	hU = MyxGraphicsMaster.metersToUnits(height),
	    	textWidth;
	    
	    //find string width so we can center it
	    textWidth = MyxGraphicsMaster.widthOfString(texRen.getFont(), String.valueOf((int)zone.score));
	    
	    gl.glDisable(GL.GL_CULL_FACE);
	    gl.glPushMatrix();	    
	    gl.glTranslated(xU, hU+0.25, zU);
	    gl.glRotated(counter, 0, 1, 0);  //counter is passed in and allows for the text spinning
	    gl.glTranslated(-(textWidth/200), 0, 0);  //Center the Text at the axis of rotation
	    texRen.setColor(0.0f, 1.0f, 1.0f, 1.0f);
	    texRen.begin3DRendering();
	    	texRen.draw3D(String.valueOf((int)zone.score), 0, 0, 0, 0.02f);
	    texRen.end3DRendering();
	    gl.glPopMatrix();
	    gl.glEnable(GL.GL_CULL_FACE);
	    
	    
	    int max = MyxGraphicsMaster.COUNTER_MAX;
	    // maps counter to oscillate between terrain height and terrain height + bounceHeight  
	    double bounceY = max/2 - Math.abs(counter - max/2);
	    bounceY /= max/2;
	    double nudge = ((zoneUnits*2.5)-(zoneUnits/2)) / SUB_RINGS;
	    bounceY = hU+nudge+(bounceY*zoneUnits);  //bounceY + fudgeFactor so ring doesn't go into terrain
	    
	    
	    drawBounceRing(zoneUnits/2, zoneUnits*2.5, RING_SIDES, SUB_RINGS, xU, bounceY, zU);
	    gl.glDisable(GL.GL_CULL_FACE);
	}
		
	/*
	 * This method draws a bouncing ring around the landing site
	 * NOTE:  code for drawing torus has been modified from an online tutorial
	 * 
	 */
	private void drawBounceRing(double r, double R, int nsides, int rings, double locX, double locY, double locZ)
	{
		
		gl.glPushMatrix();		
		gl.glTranslated(locX, locY, locZ);	
		gl.glScaled(.25, .25, .25);
	    gl.glRotated(90, 0, 0, 1);
		
		
		double ringDelta = 2.0 * Math.PI / rings;
		double sideDelta = 2.0 * Math.PI / nsides;
		double theta = 0.0f, cosTheta = 1.0f, sinTheta = 0.0f;
		
		for (int i = rings - 1; i >= 0; i--) {
		  double theta1 = theta + ringDelta;
		  double cosTheta1 = Math.cos(theta1);
		  double sinTheta1 = Math.sin(theta1);
		  
		  gl.glBegin(GL.GL_QUAD_STRIP);
		  gl.glColor4d(0.8, 0.8, 0.8, 0.3);
		  double phi = 0.0f;
		  for (int j = nsides; j >= 0; j--) {
		    phi += sideDelta;
		    double cosPhi = Math.cos(phi);
		    double sinPhi = Math.sin(phi);
		    double dist = R + r * cosPhi;
		    gl.glNormal3d(sinPhi, cosTheta1 * cosPhi, -sinTheta1 * cosPhi);
		    gl.glVertex3d(r * sinPhi, -sinTheta1 * dist, cosTheta1 * dist);
		    gl.glNormal3d(sinPhi, cosTheta * cosPhi, -sinTheta * cosPhi);
		    gl.glVertex3d(r * sinPhi, -sinTheta * dist, cosTheta * dist);
		  }
		  gl.glEnd();
		  
		  theta = theta1;
		  cosTheta = cosTheta1;
		  sinTheta = sinTheta1;
		}		
		    
		gl.glPopMatrix();
	}

	public void begin(){
		tm = (TextureIntf)MyxUtils.getFirstRequiredServiceObject(this, txtmsg);
	}

	public Object getServiceObject(IMyxName arg0) {
		// TODO Auto-generated method stub
		if (arg0.equals(msg)){
			return this;
		}
		return null;
	}

}
